print("import: 'leather'")
import leather

print("import: 'leather.scales'")
import leather.scales

print("import: 'leather.series'")
import leather.series

print("import: 'leather.shapes'")
import leather.shapes

print("import: 'leather.ticks'")
import leather.ticks

